import java.awt.*;
import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.EnumSet;
import java.util.List;
import java.util.Random;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import java.util.stream.Collectors;

import javax.swing.*;

import com.mindfusion.drawing.*;
import com.mindfusion.charting.*;
import com.mindfusion.charting.components.LayoutAlignment;
import com.mindfusion.charting.swing.*;
import com.mindfusion.common.DateTime;
import com.mindfusion.common.SwingExecutor;


public class Tutorial1
{
	Chart createChart()
	{
		BarChart chart = new BarChart();
		
		fetchStockPrices("ACME").thenAcceptAsync(
			(stockPrices) ->
			{
				// a series of daily opening prices
				SimpleSeries openSeries = new SimpleSeries(
					mapList(stockPrices, sp -> sp.getOpen()),
					mapList(stockPrices, sp -> new DateTime(sp.getDate()).toShortDateString()));
				openSeries.setTitle("Open");
				openSeries.setSupportedLabels(
					EnumSet.of(LabelKinds.XAxisLabel));
				chart.getSeries().add(openSeries);
	
				// a series of daily closing prices
				SimpleSeries closeSeries = new SimpleSeries(
					mapList(stockPrices, sp -> sp.getClose()),
					null /* no additional string labels*/);
				closeSeries.setTitle("Close");
				chart.getSeries().add(closeSeries);
	
				// a series of daily min prices
				SimpleSeries minSeries = new SimpleSeries(
					mapList(stockPrices, sp -> sp.getLow()),
					null /* no additional string labels*/);
				minSeries.setTitle("Low");
				chart.getSeries().add(minSeries);
	
				// a series of daily max prices
				SimpleSeries maxSeries = new SimpleSeries(
					mapList(stockPrices, sp -> sp.getHigh()),
					null /* no additional string labels*/);
				maxSeries.setTitle("Low");
				chart.getSeries().add(maxSeries);
			},
			new SwingExecutor());

		// assign different colors to each series
		chart.getTheme().setCommonSeriesFills(
			Colors.Yellow, Colors.LightBlue, Colors.Red, Colors.Green);

		// customize axis appearance
		chart.setTitle("ACME stock price weekly");
		chart.getYAxis().setMinValue(0.0);
		chart.getYAxis().setTitle("stock price in USD");
		chart.getXAxis().setTitle("");
		chart.setShowXCoordinates(false);
		chart.setShowXTicks(false);
		chart.setLegendHorizontalAlignment(LayoutAlignment.Far);

		return chart;
	}

	static CompletableFuture<List<StockPrice>> fetchStockPrices(String name)
	{
		// pretend we are calling a web service, returning
		// stock price statistics for last 7 days
        return CompletableFuture.supplyAsync(() ->
        {
			var random = new Random();
			Instant date = Instant.now().minus(8, ChronoUnit.DAYS);

			var prices = new ArrayList<StockPrice>();
			for (int i = 0; i < 7; i++)
			{
				date = date.plus(1, ChronoUnit.DAYS);

				double open = 200 + 500 * random.nextDouble();
				double close = open + 50 * (0.5 - random.nextDouble());
				double low = Math.min(open, close) - 50 * random.nextDouble();
				double high = Math.max(open, close) + 50 * random.nextDouble();

				prices.add(
					new StockPrice(open, close, low, high, Date.from(date)));
			}
			return prices;
        });
    }

	static <R, T> List<R> mapList(List<T> list, Function<T, R> mapper)
	{
		return list.stream().map(mapper).collect(Collectors.toList());
	}

	Tutorial1()
	{
		JFrame frame = new JFrame();
		frame.setTitle("MindFusion.Charting sample: Tutorial 1");
		frame.setSize(1000, 600);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// add chart component to the middle of the frame
		frame.getContentPane().add(
			createChart(), BorderLayout.CENTER);

		JTextArea description = new JTextArea(
			"This example demonstrates how to load data into a chart.");
		description.setEditable(false);
		frame.getContentPane().add(
			description, BorderLayout.SOUTH);

		frame.setVisible(true);
	}

	static public void main(String[] args)
	{
		SwingUtilities.invokeLater(Tutorial1::new);
	}
}
